using System.Collections.Generic;
using System.IO;
using System.IO.Compression;
using System.Net.Http;
using System.Net.Http.Headers;

namespace OSS
{
    class DownloadEvidenceSummaryAndDocuments
    {
        public static void Main(string[] args)
        {
            var API_KEY = "your_api_key";
            var API_URL = "https://sandbox.esignlive.com/api";
            var packageId = "c1cQItVo-DG3xuLBgUKCqBBMAC0=";
            var outputPath = $"C:\\...\\{packageId}.zip";

            byte[] zipByteArray = DownloadSignedDocument($"{API_URL}/packages/{packageId}/documents/zip", API_KEY);
            byte[] pdf1 = DownloadSignedDocument($"{API_URL}/packages/{packageId}/evidence/summary", API_KEY);

            Dictionary<string, byte[]> extractedPDFs = ExtractPDFsFromZip(zipByteArray);
            extractedPDFs.Add("Evidence Summary.pdf", pdf1);

            byte[] newZipByteArray = CreateZipFromPDFs(extractedPDFs);

            using (FileStream fs = new FileStream(outputPath, FileMode.Create))
            {
                fs.Write(newZipByteArray, 0, newZipByteArray.Length);
            }
        }

        public static Dictionary<string, byte[]> ExtractPDFsFromZip(byte[] zipByteArray)
        {
            Dictionary<string, byte[]> pdfs = new Dictionary<string, byte[]>();

            using (MemoryStream ms = new MemoryStream(zipByteArray))
            using (ZipArchive archive = new ZipArchive(ms))
            {
                foreach (ZipArchiveEntry entry in archive.Entries)
                {
                    if (entry.Name.ToLower().EndsWith(".pdf"))
                    {
                        using (MemoryStream pdfStream = new MemoryStream())
                        {
                            using (Stream entryStream = entry.Open())
                            {
                                entryStream.CopyTo(pdfStream);
                            }
                            pdfs.Add(entry.Name, pdfStream.ToArray());
                        }
                    }
                }
            }
            return pdfs;
        }

        public static byte[] CreateZipFromPDFs(Dictionary<string, byte[]> pdfs)
        {
            using (MemoryStream ms = new MemoryStream())
            {
                using (ZipArchive archive = new ZipArchive(ms, ZipArchiveMode.Create))
                {
                    foreach (KeyValuePair<string, byte[]> entry in pdfs)
                    {
                        ZipArchiveEntry zipEntry = archive.CreateEntry(entry.Key);

                        using (Stream entryStream = zipEntry.Open())
                        {
                            entryStream.Write(entry.Value, 0, entry.Value.Length);
                        }
                    }
                }
                return ms.ToArray();
            }
        }


        public static byte[] DownloadSignedDocument(string apiUrl, string apiKey)
        {
            using (HttpClient httpClient = new HttpClient())
            {
                httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Basic", apiKey);
                var response = httpClient.GetAsync(apiUrl).Result;
                response.EnsureSuccessStatusCode();               
                byte[] fileBytes = response.Content.ReadAsByteArrayAsync().Result;
                return fileBytes;
            }
        }


    }
}